@@ -24,6 +24,7 @@
 #                 Christopher Aillon <christopher@aillon.com>
 #                 Tomas Kopal <Tomas.Kopal@altap.cz>
 #                 Max Kanat-Alexander <mkanat@bugzilla.org>
+#                 Lance Larsh <lance.larsh@oracle.com>
 
 package Bugzilla::DB;
 
@@ -50,7 +51,6 @@ use Bugzilla::Config qw(:DEFAULT :db);
 use Bugzilla::Util;
 use Bugzilla::Error;
 use Bugzilla::DB::Schema;
-use Bugzilla::User;
 
 #####################################################################
 # Constants
@@ -128,12 +128,12 @@ sub FetchOneColumn {
     return $row[0];
 }
 
-sub PushGlobalSQLState() {
+sub PushGlobalSQLState {
     push @SQLStateStack, $_current_sth;
     push @SQLStateStack, $_fetchahead;
 }
 
-sub PopGlobalSQLState() {
+sub PopGlobalSQLState {
     die ("PopGlobalSQLState: stack underflow") if ( scalar(@SQLStateStack) < 1 );
     $_fetchahead = pop @SQLStateStack;
     $_current_sth = pop @SQLStateStack;
@@ -153,7 +153,7 @@ sub connect_shadow {
                     Param("shadowdbsock"), $db_user, $db_pass);
 }
 
-sub connect_main (;$) {
+sub connect_main {
     my ($no_db_name) = @_;
     my $connect_to_db = $db_name;
     $connect_to_db = "" if $no_db_name;
@@ -260,22 +260,26 @@ sub sql_fulltext_search {
 
     # This is as close as we can get to doing full text search using
     # standard ANSI SQL, without real full text search support. DB specific
-    # modules shoud override this, as this will be always much slower.
-
-    # the text is already sql-quoted, so we need to remove the quotes first
-    my $quote = substr($self->quote(''), 0, 1);
-    $text = $1 if ($text =~ /^$quote(.*)$quote$/);
+    # modules should override this, as this will be always much slower.
 
     # make the string lowercase to do case insensitive search
     my $lower_text = lc($text);
 
-    # split the text we search for to separate words
+    # split the text we search for into separate words
     my @words = split(/\s+/, $lower_text);
 
-    # search for occurence of all specified words in the column
-    return "CASE WHEN (LOWER($column) LIKE ${quote}%" .
-           join("%${quote} AND LOWER($column) LIKE ${quote}%", @words) .
-           "%${quote}) THEN 1 ELSE 0 END";
+    # surround the words with wildcards and SQL quotes so we can use them
+    # in LIKE search clauses
+    @words = map($self->quote("%$_%"), @words);
+
+    # untaint words, since they are safe to use now that we've quoted them
+    map(trick_taint($_), @words);
+
+    # turn the words into a set of LIKE search clauses
+    @words = map("LOWER($column) LIKE $_", @words);
+
+    # search for occurrences of all specified words in the column
+    return "CASE WHEN (" . join(" AND ", @words) . ") THEN 1 ELSE 0 END";
 }
 
 #####################################################################
@@ -298,7 +302,7 @@ sub bz_get_field_defs {
     my ($self) = @_;
 
     my $extra = "";
-    if (!UserInGroup(Param('timetrackinggroup'))) {
+    if (!Bugzilla->user->in_group(Param('timetrackinggroup'))) {
         $extra = "AND name NOT IN ('estimated_time', 'remaining_time', " .
                  "'work_time', 'percentage_complete', 'deadline')";
     }
@@ -332,6 +336,13 @@ sub bz_setup_database {
     }
 }
 
+# The defauly implementation just returns what you passed-in. This function
+# really exists just to be overridden in Bugzilla::DB::Mysql.
+sub bz_enum_initial_values {
+    my ($self, $enum_defaults) = @_;
+    return $enum_defaults;
+}
+
 #####################################################################
 # Schema Modification Methods
 #####################################################################
@@ -654,7 +665,7 @@ sub bz_table_info {
 
 sub bz_table_columns {
     my ($self, $table) = @_;
-    return $self->_bz_schema->get_table_columns($table);
+    return $self->_bz_real_schema->get_table_columns($table);
 }
 
 #####################################################################
@@ -757,7 +768,7 @@ sub db_new {
     my $self = DBI->connect($dsn, $user, $pass, $attributes)
         or die "\nCan't connect to the database.\nError: $DBI::errstr\n"
         . "  Is your database installed and up and running?\n  Do you have"
-        . "the correct username and password selected in localconfig?\n\n";
+        . " the correct username and password selected in localconfig?\n\n";
 
     # RaiseError was only set to 0 so that we could catch the 
     # above "die" condition.
@@ -1094,7 +1105,8 @@ formatted SQL command have prefix C<sql_>. All other methods have prefix C<bz_>.
               searches (case insensitive) in format suitable for a given
               database.
               Abstract method, should be overriden by database specific code.
- Params:      none
+ Params:      $expr = SQL expression for the text to be searched (scalar)
+              $pattern = the regular expression to search for (scalar)
  Returns:     formatted SQL for regular expression search (e.g. REGEXP)
               (scalar)
 
@@ -1104,7 +1116,8 @@ formatted SQL command have prefix C<sql_>. All other methods have prefix C<bz_>.
               regex searches (case insensitive) in format suitable for a given
               database.
               Abstract method, should be overriden by database specific code.
- Params:      none
+ Params:      $expr = SQL expression for the text to be searched (scalar)
+              $pattern = the regular expression to search for (scalar)
  Returns:     formatted SQL for negative regular expression search
               (e.g. NOT REGEXP) (scalar)
 
@@ -1118,6 +1131,13 @@ formatted SQL command have prefix C<sql_>. All other methods have prefix C<bz_>.
  Returns:     formatted SQL for limiting number of rows returned from query
               with optional offset (e.g. LIMIT 1, 1) (scalar)
 
+=item C<sql_from_days>
+
+ Description: Outputs SQL syntax for converting Julian days to date.
+              Abstract method, should be overriden by database specific code.
+ Params:      $days = days to convert to date
+ Returns:     formatted SQL for returning Julian days in dates. (scalar)
+
 =item C<sql_to_days>
 
  Description: Outputs SQL syntax for converting date to Julian days.
@@ -1140,8 +1160,8 @@ formatted SQL command have prefix C<sql_>. All other methods have prefix C<bz_>.
 
  Description: Outputs proper SQL syntax for a time interval function.
               Abstract method, should be overriden by database specific code.
- Params:      $interval = the time interval requested (e.g. '30 minutes')
-              (scalar)
+ Params:      $interval - the time interval requested (e.g. '30') (integer)
+              $units    - the units the interval is in (e.g. 'MINUTE') (string)
  Returns:     formatted SQL for interval function (scalar)
 
 =item C<sql_position>
@@ -1182,12 +1202,12 @@ formatted SQL command have prefix C<sql_>. All other methods have prefix C<bz_>.
               specified text on a given column.
               There is a ANSI SQL version of this method implemented using
               LIKE operator, but it's not a real full text search. DB specific
-              modules shoud override this, as this generic implementation will
+              modules should override this, as this generic implementation will
               be always much slower. This generic implementation returns
               'relevance' as 0 for no match, or 1 for a match.
  Params:      $column = name of column to search (scalar)
               $text = text to search for (scalar)
- Returns:     formatted SQL for for full text search
+ Returns:     formatted SQL for full text search
 
 =item C<sql_istrcmp>
 
@@ -1270,6 +1290,26 @@ These methods return information about data in the database.
 =back
 
 
+=head2 Database Setup Methods
+
+These methods are used by the Bugzilla installation programs to set up
+the database.
+
+=over 4
+
+=item C<bz_enum_initial_values(\%enum_defaults)>
+
+ Description: For an upgrade or an initial installation, provides
+              what the values should be for the "enum"-type fields,
+              such as version, op_sys, rep_platform, etc.
+ Params:      \%enum_defaults - The default initial list of values for
+                                each enum field. A hash, with the field
+                                names pointing to an arrayref of values.
+ Returns:     A hashref with the correct initial values for the enum fields.
+
+=back
+
+
 =head2 Schema Modification Methods
 
 These methods modify the current Bugzilla Schema.
