@@ -25,13 +25,11 @@
 #                 Myk Melez <myk@mozilla.org>
 #                 Michael Schindler <michael@compressconsult.com>
 #                 Max Kanat-Alexander <mkanat@bugzilla.org>
+#                 Joel Peshkin <bugreport@peshkin.net>
+#                 Lance Larsh <lance.larsh@oracle.com>
 
 use strict;
 
-# The caller MUST require CGI.pl and globals.pl before using this
-
-use vars qw($userid);
-
 package Bugzilla::Search;
 use base qw(Exporter);
 @Bugzilla::Search::EXPORT = qw(IsValidQueryType);
@@ -42,6 +40,7 @@ use Bugzilla::Util;
 use Bugzilla::Constants;
 use Bugzilla::Group;
 use Bugzilla::User;
+use Bugzilla::Field;
 
 use Date::Format;
 use Date::Parse;
@@ -99,6 +98,8 @@ sub init {
     my @orderby;
 
     my $debug = 0;
+    my @debugdata;
+    if ($params->param('debug')) { $debug = 1; }
 
     my @fields;
     my @supptables;
@@ -310,7 +311,7 @@ sub init {
                     push(@l, "bugs.creation_ts <= $sql_chto") if($sql_chto);
                     $bug_creation_clause = "(" . join(' AND ', @l) . ")";
                 } else {
-                    push(@list, "\nactcheck.fieldid = " . &::GetFieldID($f));
+                    push(@list, "\nactcheck.fieldid = " . get_field_id($f));
                 }
             }
 
@@ -349,14 +350,18 @@ sub init {
             
       if ($params->param('deadlinefrom')){
         $deadlinefrom = $params->param('deadlinefrom');
-        Bugzilla::Util::ValidateDate($deadlinefrom, 'deadlinefrom');
+        validate_date($deadlinefrom)
+          || ThrowUserError('illegal_date', {date => $deadlinefrom,
+                                             format => 'YYYY-MM-DD'});
         $sql_deadlinefrom = &::SqlQuote($deadlinefrom);
         push(@wherepart, "bugs.deadline >= $sql_deadlinefrom");
       }
       
       if ($params->param('deadlineto')){
         $deadlineto = $params->param('deadlineto');
-        Bugzilla::Util::ValidateDate($deadlineto, 'deadlineto');
+        validate_date($deadlineto)
+          || ThrowUserError('illegal_date', {date => $deadlineto,
+                                             format => 'YYYY-MM-DD'});
         $sql_deadlineto = &::SqlQuote($deadlineto);
         push(@wherepart, "bugs.deadline <= $sql_deadlineto");
       }
@@ -427,9 +432,14 @@ sub init {
              $term = "bugs.$f <> " . pronoun($1, $user);
           },
          "^(assigned_to|reporter),(?!changed)" => sub {
-             push(@supptables, "INNER JOIN profiles AS map_$f " .
-                               "ON bugs.$f = map_$f.userid");
-             $f = "map_$f.login_name";
+             my $list = $self->ListIDsForEmail($t, $v);
+             if ($list) {
+                 $term = "bugs.$f IN ($list)"; 
+             } else {
+                 push(@supptables, "INNER JOIN profiles AS map_$f " .
+                                   "ON bugs.$f = map_$f.userid");
+                 $f = "map_$f.login_name";
+             }
          },
          "^qa_contact,(?!changed)" => sub {
              push(@supptables, "LEFT JOIN profiles AS map_qa_contact " .
@@ -489,7 +499,7 @@ sub init {
                                "AND cc_$chartseq.who = $match");
              $term = "cc_$chartseq.who IS NULL";
          },
-         "^cc,(anyexact|substring)" => sub {
+         "^cc,(anyexact|substring|regexp)" => sub {
              my $list;
              $list = $self->ListIDsForEmail($t, $v);
              my $chartseq = $chartid;
@@ -576,10 +586,8 @@ sub init {
              # $term1 searches comments.
              # $term2 searches summaries, which contributes to the relevance
              # ranking in SELECT but doesn't limit which bugs get retrieved.
-             my $term1 = $dbh->sql_fulltext_search("${table}.thetext",
-                                                   ::SqlQuote($v));
-             my $term2 = $dbh->sql_fulltext_search("bugs.short_desc",
-                                                   ::SqlQuote($v));
+             my $term1 = $dbh->sql_fulltext_search("${table}.thetext", $v);
+             my $term2 = $dbh->sql_fulltext_search("bugs.short_desc", $v);
 
              # The term to use in the WHERE clause.
              $term = "$term1 > 0";
@@ -716,14 +724,23 @@ sub init {
              } elsif ($t eq "notequal") {
                  $oper = "<>";
              } elsif ($t eq "regexp") {
-                 $oper = $dbh->sql_regexp();
+                 # This is just a dummy to help catch bugs- $oper won't be used
+                 # since "regexp" is treated as a special case below.  But
+                 # leaving $oper uninitialized seems risky...
+                 $oper = "sql_regexp";
              } elsif ($t eq "notregexp") {
-                 $oper = $dbh->sql_not_regexp();
+                 # This is just a dummy to help catch bugs- $oper won't be used
+                 # since "notregexp" is treated as a special case below.  But
+                 # leaving $oper uninitialized seems risky...
+                 $oper = "sql_not_regexp";
              } else {
                  $oper = "noop";
              }
              if ($oper ne "noop") {
                  my $table = "longdescs_$chartid";
+                 if(lsearch(\@fields, "bugs.remaining_time") == -1) {
+                     push(@fields, "bugs.remaining_time");                  
+                 }
                  push(@supptables, "INNER JOIN longdescs AS $table " .
                                    "ON $table.bug_id = bugs.bug_id");
                  my $expression = "(100 * ((SUM($table.work_time) *
@@ -733,7 +750,13 @@ sub init {
                                               COUNT(DISTINCT $table.bug_when) /
                                               COUNT(bugs.bug_id)) +
                                              bugs.remaining_time)))";
-                 push(@having, "$expression $oper " . &::SqlQuote($v));
+                 if ($t eq "regexp") {
+                     push(@having, $dbh->sql_regexp($expression, &::SqlQuote($v)));
+                 } elsif ($t eq "notregexp") {
+                     push(@having, $dbh->sql_not_regexp($expression, &::SqlQuote($v)));
+                 } else {
+                     push(@having, "$expression $oper " . &::SqlQuote($v));
+                 }
                  push(@groupby, "bugs.remaining_time");
              }
              $term = "0=0";
@@ -742,11 +765,32 @@ sub init {
             push(@supptables,
                     "LEFT JOIN bug_group_map AS bug_group_map_$chartid " .
                     "ON bugs.bug_id = bug_group_map_$chartid.bug_id");
-
+            $ff = $f = "groups_$chartid.name";
+            my $ref = $funcsbykey{",$t"};
+            &$ref;
             push(@supptables,
                     "LEFT JOIN groups AS groups_$chartid " .
-                    "ON groups_$chartid.id = bug_group_map_$chartid.group_id");
-            $f = "groups_$chartid.name";
+                    "ON groups_$chartid.id = bug_group_map_$chartid.group_id " .
+                    "AND $term");
+            $term = "$ff IS NOT NULL";
+         },
+         "^attach_data\.thedata,changed" => sub {
+            # Searches for attachment data's change must search
+            # the creation timestamp of the attachment instead.
+            $f = "attachments.whocares";
+         },
+         "^attach_data\.thedata," => sub {
+             my $atable = "attachments_$chartid";
+             my $dtable = "attachdata_$chartid";
+             my $extra = "";
+             if (Param("insidergroup") && !UserInGroup(Param("insidergroup"))) {
+                 $extra = "AND $atable.isprivate = 0";
+             }
+             push(@supptables, "INNER JOIN attachments AS $atable " .
+                               "ON bugs.bug_id = $atable.bug_id $extra");
+             push(@supptables, "INNER JOIN attach_data AS $dtable " .
+                               "ON $dtable.id = $atable.attach_id");
+             $f = "$dtable.thedata";
          },
          "^attachments\..*," => sub {
              my $table = "attachments_$chartid";
@@ -844,7 +888,7 @@ sub init {
              $f = "setters_$chartid.login_name";
          },
          
-         "^changedin," => sub {
+         "^(changedin|days_elapsed)," => sub {
              $f = "(" . $dbh->sql_to_days('NOW()') . " - " .
                         $dbh->sql_to_days('bugs.delta_ts') . ")";
          },
@@ -960,8 +1004,8 @@ sub init {
                     $unitinterval = 'YEAR';
                 }
                 my $cutoff = "NOW() - " .
-                             $dbh->sql_interval("$quantity $unitinterval");
-                my $assigned_fieldid = &::GetFieldID('assigned_to');
+                             $dbh->sql_interval($quantity, $unitinterval);
+                my $assigned_fieldid = get_field_id('assigned_to');
                 push(@supptables, "LEFT JOIN longdescs AS comment_$table " .
                                   "ON comment_$table.who = bugs.assigned_to " .
                                   "AND comment_$table.bug_id = bugs.bug_id " .
@@ -1000,10 +1044,10 @@ sub init {
              $term = $dbh->sql_position(lc($q), "LOWER($ff)") . " = 0";
          },
          ",regexp" => sub {
-             $term = "$ff " . $dbh->sql_regexp() . " $q";
+             $term = $dbh->sql_regexp($ff, $q);
          },
          ",notregexp" => sub {
-             $term = "$ff " . $dbh->sql_not_regexp() . " $q";
+             $term = $dbh->sql_not_regexp($ff, $q);
          },
          ",lessthan" => sub {
              $term = "$ff < $q";
@@ -1122,7 +1166,9 @@ sub init {
             $params->param("type$chart-$row-$col", shift(@$ref));
             $params->param("value$chart-$row-$col", shift(@$ref));
             if ($debug) {
-                print qq{<p>$params->param("field$chart-$row-$col") | $params->param("type$chart-$row-$col") | $params->param("value$chart-$row-$col")*</p>\n};
+                push(@debugdata, "$row-$col = " .
+                               $params->param("field$chart-$row-$col") . ' | ' .                               $params->param("type$chart-$row-$col") . ' | ' .
+                               $params->param("value$chart-$row-$col") . ' *');
             }
             $col++;
 
@@ -1260,7 +1306,7 @@ sub init {
                     if ("$f,$t,$rhs" =~ m/$key/) {
                         my $ref = $funcsbykey{$key};
                         if ($debug) {
-                            print "<p>$key ($f , $t , $rhs ) => ";
+                            push(@debugdata, "$key ($f / $t / $rhs) =>");
                         }
                         $ff = $f;
                         if ($f !~ /\./) {
@@ -1268,7 +1314,8 @@ sub init {
                         }
                         &$ref;
                         if ($debug) {
-                            print "$f , $t , $v , $term</p>";
+                            push(@debugdata, "$f / $t / $v / " .
+                                             ($term || "undef") . " *");
                         }
                         if ($term) {
                             last;
@@ -1379,11 +1426,12 @@ sub init {
     foreach my $field (@fields, @orderby) {
         next if ($field =~ /(AVG|SUM|COUNT|MAX|MIN|VARIANCE)\s*\(/i ||
                  $field =~ /^\d+$/ || $field eq "bugs.bug_id" ||
-                 $field =~ /^relevance/);
-        if ($field =~ /.*AS\s+(\w+)$/i) {
+                 $field =~ /^(relevance|actual_time|percentage_complete)/);
+        # The structure of fields is of the form:
+        # [foo AS] {bar | bar.baz} [ASC | DESC]
+        # Only the mandatory part bar OR bar.baz is of interest
+        if ($field =~ /(?:.*\s+AS\s+)?(\w+(\.\w+)?)(?:\s+(ASC|DESC))?$/i) {
             push(@groupby, $1) if !grep($_ eq $1, @groupby);
-        } else {
-            push(@groupby, $field) if !grep($_ eq $field, @groupby);
         }
     }
     $query .= ") " . $dbh->sql_group_by("bugs.bug_id", join(', ', @groupby));
@@ -1397,12 +1445,8 @@ sub init {
         $query .= " ORDER BY " . join(',', @orderby);
     }
 
-    if ($debug) {
-        print "<p><code>" . value_quote($query) . "</code></p>\n";
-        exit;
-    }
-    
     $self->{'sql'} = $query;
+    $self->{'debugdata'} = \@debugdata;
 }
 
 ###############################################################################
@@ -1417,7 +1461,7 @@ sub SqlifyDate {
     }
 
 
-    if ($str =~ /^(-|\+)?(\d+)([dDwWmMyY])$/) {   # relative date
+    if ($str =~ /^(-|\+)?(\d+)([hHdDwWmMyY])$/) {   # relative date
         my ($sign, $amount, $unit, $date) = ($1, $2, lc $3, time);
         my ($sec, $min, $hour, $mday, $month, $year, $wday)  = localtime($date);
         if ($sign && $sign eq '+') { $amount = -$amount; }
@@ -1437,6 +1481,15 @@ sub SqlifyDate {
             while ($month<0) { $year--; $month += 12; }
             return sprintf("%4d-%02d-01 00:00:00", $year+1900, $month+1);
         }
+        elsif ($unit eq 'h') {
+            # Special case 0h for 'beginning of this hour'
+            if ($amount == 0) {
+                $date -= $sec + 60*$min;
+            } else {
+                $date -= 3600*$amount;
+            }
+            return time2str("%Y-%m-%d %H:%M:%S", $date);
+        }
         return undef;                      # should not happen due to regexp at top
     }
     my $date = str2time($str);
@@ -1449,8 +1502,8 @@ sub SqlifyDate {
 # ListIDsForEmail returns a string with a comma-joined list
 # of userids matching email addresses
 # according to the type specified.
-# Currently, this only supports exact, anyexact, and substring matches.
-# Substring matches will return up to 50 matching userids
+# Currently, this only supports regexp, exact, anyexact, and substring matches.
+# Matches will return up to 50 matching userids
 # If a match type is unsupported or returns too many matches,
 # ListIDsForEmail returns an undef.
 sub ListIDsForEmail {
@@ -1479,7 +1532,18 @@ sub ListIDsForEmail {
             my ($id) = &::FetchSQLData();
             push(@list, $id);
         }
-        if (@list < 50) {
+        if (scalar(@list) < 50) {
+            $list = join(',', @list);
+        }
+    } elsif ($type eq 'regexp') {
+        &::SendSQL("SELECT userid FROM profiles WHERE " .
+            $dbh->sql_regexp("login_name", ::SqlQuote($email)) .
+            " " . $dbh->sql_limit(51));
+        while (&::MoreSQLData()) {
+            my ($id) = &::FetchSQLData();
+            push(@list, $id);
+        }
+        if (scalar(@list) < 50) {
             $list = join(',', @list);
         }
     }
@@ -1514,7 +1578,7 @@ sub GetByWordList {
             $word =~ s/^'//;
             $word =~ s/'$//;
             $word = '(^|[^a-z0-9])' . $word . '($|[^a-z0-9])';
-            push(@list, "$field " . $dbh->sql_regexp() . " '$word'");
+            push(@list, $dbh->sql_regexp($field, "'$word'"));
         }
     }
 
@@ -1542,6 +1606,11 @@ sub getSQL {
     return $self->{'sql'};
 }
 
+sub getDebugData {
+    my $self = shift;
+    return $self->{'debugdata'};
+}
+
 sub pronoun {
     my ($noun, $user) = (@_);
     if ($noun eq "%user%") {
@@ -1624,6 +1693,6 @@ sub BuildOrderBy {
         return;
     }
 
-    push(@$stringlist, $orderfield . ' ' . $orderdirection);
+    push(@$stringlist, trim($orderfield . ' ' . $orderdirection));
 }
 1;
