@@ -100,6 +100,7 @@
 
 'reports/components.html.tmpl' => [
   'numcols',
+  'comp.description', 
 ],
 
 'reports/duplicates-table.html.tmpl' => [
@@ -118,6 +119,7 @@
 ],
 
 'reports/keywords.html.tmpl' => [
+  'keyword.description', 
   'keyword.bugcount', 
 ],
 
@@ -196,10 +198,16 @@
 
 'list/edit-multiple.html.tmpl' => [
   'group.id', 
+  'group.description',
+  'group.description FILTER inactive', 
   'knum', 
   'menuname', 
 ],
 
+'list/list-simple.html.tmpl' => [
+  'title', 
+],
+
 'list/list.rdf.tmpl' => [
   'template_version', 
   'bug.bug_id', 
@@ -230,8 +238,13 @@
   'VERSION', 
 ],
 
+'global/choose-classification.html.tmpl' => [
+  'class.description', 
+],
+
 'global/choose-product.html.tmpl' => [
   'target',
+  'proddesc.$p', 
 ],
 
 # You are not permitted to add any values here. Everything in this file should 
@@ -244,7 +257,6 @@
   'javascript', 
   'style', 
   'onload',
-  'title',
   'h1',
   'h2',
   'h3', 
@@ -318,11 +330,14 @@
   'bug.bug_id', 
   'bug.votes', 
   'group.bit', 
+  'group.description', 
   'dep.title', 
   'dep.fieldname', 
   'accesskey', 
   'bug.${dep.fieldname}.join(\', \')', 
   'selname',
+  'depbug FILTER bug_link(depbug)',
+  '"${bug.dup_id}" FILTER bug_link(bug.dup_id)',
 ],
 
 'bug/knob.html.tmpl' => [
@@ -397,6 +412,7 @@
 
 'bug/create/create.html.tmpl' => [
   'g.bit',
+  'g.description',
   'sel.name',
   'sel.description',
   'cloned_bug_id'
@@ -443,7 +459,7 @@
 ],
 
 'attachment/show-multiple.html.tmpl' => [
-  'a.id',
+  'a.attachid', 
   'flag.status'
 ],
 
@@ -476,6 +492,10 @@
   'link_uri'
 ],
 
+'admin/classifications/select.html.tmpl' => [
+  'cl.description', 
+],
+
 'admin/products/groupcontrol/confirm-edit.html.tmpl' => [
   'group.count', 
 ],
@@ -529,6 +549,7 @@
 
 'admin/flag-type/list.html.tmpl' => [
   'type.id', 
+  'type.flag_count', 
 ],
 
 
@@ -582,6 +603,11 @@
   'bug_count'
 ],
 
+'admin/settings/edit.html.tmpl' => [
+  'name',
+  'checkbox_name'
+],
+
 'account/login.html.tmpl' => [
   'target', 
 ],
@@ -592,9 +618,19 @@
   'prefname',
 ],
 
+'account/prefs/permissions.html.tmpl' => [
+  'bit_description.name', 
+  'bit_description.desc', 
+],
+
 'account/prefs/prefs.html.tmpl' => [
   'current_tab.label',
   'current_tab.name',
 ],
 
+'account/prefs/settings.html.tmpl' => [
+  'name',
+  'default_name'
+],
+
 );
